(* ©Harald E Brandt. You may use any of this code for any purpose you like.
It is meant to fix old Mori web links that somehow got lost. I.e things like http://... that once were links but now are just text colored in blue or similar but the actual link is gone. (Perhaps some kind of corruptness).
It basically works as follows:
It iterates through all rtf and rtfd files in a database named "Notebase" and searches for all attribute runs that start with "http://". Then it sets the link URL (from missing value) to the text of the attribute run after trailing white space and linefeeds are strippe. However, before it set that it checks for 2 problems: (1) it checks if there is a linefeed in the string, and if it is, then the attribute run is a regular text string that just happens to start with http:// and in this case it shows a display dialog for 2 seconds and the proceeds (i.e it skips this item); (2) it when it attempts to set the url, then the corresponding item in the view window must match the item found in the rtf (see explanation in the comments) and if it does not do that (i.e a mismatch) then it shows a display dialog for 2 seconds and the proceeds (i.e it skips this item).
After the change described above, it will first save the file and then set the modification date to what it was before the change, i.e all modification dates are preserved.
In the end, it will report the result of the number of files changed, the number of changed links. In my case, the following was the result of running it in AppleScriptObjC Exlorer:
000,0010 Result
--> Number of changes:
Number of changed files: 177
Number of changed links: 363
Number of texts that start out as an URL but include a linefeed in the text: 61
Number of attribute runs in the rtf that do not match the view in the window: 0
000,0002 -- End -- (~176,144s)
*)
set nrOfChangedFiles to 0
set nrOfChangedLinks to 0
set nrOfImpoperUrls to 0
set nrOfMismatches to 0
tell application "DEVONthink Pro"
	repeat with aGroup in parents of database "Notebase"
		if kind of aGroup is "Group" and name of aGroup is not in {"Trash", "Inbox", "Tags", "Mobile Sync"} then
			repeat with anRTF in (children of aGroup whose kind is "RTFD" or kind is "RTF")
				--repeat with anRTF in (children of aGroup whose (kind is "RTFD" or kind is "RTF") and name is "Nya securities?")
				set anRTFisChanged to false --a flag so we can count the number of files we changed
				set allURLs to URL of attribute runs of rich text of anRTF --list of URLs, possibly with missing value
				set allRuns to attribute runs of rich text of anRTF
				repeat with i from 1 to count allRuns
					set theText to text of item i of allRuns
					if item i of allURLs is missing value and length of theText ≥ 7 and texts 1 thru 7 of theText is "http://" then
						-- For some inexplicable weird reason, "item i of attribute runs of …" is NOT always the same item as "attribute run i of …"!!? It should be, but for some longer texts in some files, the same item can have different "i" indexes! (By 1 or perhaps 2 units.) Weird bug, so we have to be very consistent only doing "item i of ..."
						-- To set the URL to a new value, we can't do that on that on theText, and not even on the attribute run of anRTF, since that is data rather than pointers into the actual document. Instead we have to target the attribute run of the text of a viewer window. However, the next weird bug is that "item i of attribute runs of text of viewer window 1" is NOT always the same item as "item i of attribute runs of rich text of anRTF"!!? Again, the same item can have different "i" indexes! We will check that they are equal and display a dialog if they are not.
						set selection of viewer window 1 to {anRTF}
						set allViewURLs to URL of attribute runs of text of viewer window 1 --list of URLs or 'missing value'
						set allViewTexts to text of attribute runs of text of viewer window 1
						if item i of allViewURLs is missing value and item i of allViewTexts is theText then
							set theViewRun to (a reference to item i of attribute runs of text of viewer window 1) --this is a reference to the attribute run in the window.
							-- Strip white space from theText since we don't want any trailing spaces or LF in an URL:
							set theText to current application's NSString's stringWithString_(theText)'s stringByTrimmingCharactersInSet_(current application's NSCharacterSet's whitespaceAndNewlineCharacterSet) as text
							if linefeed is in theText then
								tell me to display dialog "Not a proper URL." & return & "We will skip this one." buttons {"Cancel", "Skip"} default button 2 giving up after 2
								set nrOfImpoperUrls to nrOfImpoperUrls + 1
							else
								set modDate to modification date of anRTF
								set URL of theViewRun to theText
								save viewer window 1
								set modification date of anRTF to modDate
								set nrOfChangedLinks to nrOfChangedLinks + 1
								tell me to log "nrOfChangedLinks: " & nrOfChangedLinks --good to know if we want to cancel
								set anRTFisChanged to true
							end if
						else
							tell me to display dialog "Oh Shit!" & return & "There is a mismatch between the window view and the rtf file. We will skip this one" buttons {"Cancel", "Skip"} default button 2 giving up after 2
							set nrOfMismatches to nrOfMismatches + 1
						end if
					end if
				end repeat
				if anRTFisChanged then
					set nrOfChangedFiles to nrOfChangedFiles + 1
					tell me to log "nrOfChangedFiles: " & nrOfChangedFiles --good to know if we want to cancel
				end if
			end repeat
		end if
	end repeat
end tell
return "Number of changes:" & return & ¬
	"Number of changed files: " & nrOfChangedFiles & return & ¬
	"Number of changed links: " & nrOfChangedLinks & return & ¬
	"Number of texts that start out as an URL but include a linefeed in the text: " & nrOfImpoperUrls & return & ¬
	"Number of attribute runs in the rtf that do not match the view in the window: " & nrOfMismatches