(* ©Harald E Brandt. You may use any of this code for any purpose you like.
It is meant to change old Mori internal links so that they point to the corresponding file in DEVONthink Pro instead. You should first run the script "Mori same-group links in Notebase" that fixes most links to files that belong to the same group (reside in the same folder) as the file that contains the link. After that script is run, this script can be run to fix the links that point to files in some other group, or links that the other script couldn't fix automatically. Since it is not possible to unambiguously find which file that shall be the target of a link, this script is semi-automatic so that it often proposes a file, displays a dialog, and then you can ether just proceed with the selected file or select another file before you proceed.
It basically works as follows:
It iterates through all rtf and rtfd files in a database named "Notebase" and searches for all link URLs that start with "mori://", then it searches the entire database for a file that has the same name as the text of the link minus any leading or trailing white space or linefeeds. If such a file is found, it is selected, and a dialog is displayed where you can see the name. You may accept the selected file or select a new one. Then it sets the link URL to the selected file (i.e the new link URL will be something like: "x-devonthink-item://162E0728-9051-4EBC-8C85-35D809D3A665").
After the change described above, it will first save the file and then set the modification date to what it was before the change, i.e all modification dates are preserved.
In the end, it will report the result of the number of files changed, the number of changed links. In my case, the following was the result of running it in AppleScriptObjC Exlorer:
--> Number of changes:
Number of changed files: 144
Number of changed links: 225
It took me a total of about 2 hours!
Since it may happen that you have to cancel to fix things, or it bails out or whatever, it will continuously log the number of changed files and links, so when you continue you can later add up the number.
*)
set nrOfChangedFiles to 0
set nrOfChangedLinks to 0
tell application "DEVONthink Pro"
	repeat with aGroup in parents of database "Notebase"
		if kind of aGroup is "Group" and name of aGroup is not in {"Trash", "Inbox", "Tags", "Mobile Sync"} then
			repeat with anRTF in (children of aGroup whose kind is "RTFD" or kind is "RTF")
				set anRTFisChanged to false --a flag so we can count the number of files we changed
				set allURLs to URL of attribute runs of rich text of anRTF --list of URLs, possibly with "missing value"
				repeat with i from 1 to count allURLs --note that I can't use "contents of ..." because conflict of terms!!
					set anURL to item i of allURLs
					if anURL is not missing value and texts 1 thru 7 of anURL is "mori://" then
						set theText to text of item i of attribute runs of rich text of anRTF
						-- For some inexplicable weird reason, "item i of attribute runs of …" is NOT always the same item as "attribute run i of …"!!? It should be, but for some longer texts in some files, the same item can have different "i" indexes! (By 1 or perhaps 2 units.) Weird bug, so we have to be very consistent only doing "item i of ..."
						-- To set the URL to a new value, we can't do that on theText, and not even on the attribute run of anRTF, since that is data rather than pointers into the actual document. Instead we have to target the attribute run of the text of a viewer window. However, the next weird bug is that "item i of attribute runs of text of viewer window 1" is NOT always the same item as "item i of attribute runs of rich text of anRTF"!!? Again, the same item can have different "i" indexes! It appears that the ONLY way to get the right item is to do a search for that item! Scary, but if we search for both its text and URL I hope we are safe...
						set selection of viewer window 1 to {anRTF}
						--delay 2 --so as to see the file we will modify before we propose a target for the link (questionable thing)
						set allViewURLs to URL of attribute runs of text of viewer window 1 --list of URLs or 'missing value'
						set allViewTexts to text of attribute runs of text of viewer window 1
						set theViewRun to missing value --just in case something is wrong, we will check if it remains missing value (should never happen)
						repeat with j from 1 to count allViewURLs
							-- It does NOT work with "URL of item j of attribute runs of text of viewer window 1 is …" if it is missing value, but it DOES work with "item j of URL of attribute runs of text of viewer window 1 is …"! Weird!! That's why this repeat loop has to use explicit indexing rather than "fast iteration" using "repeat with aViewRun in …"
							if item j of allViewURLs is anURL and item j of allViewTexts is theText then
								set theViewRun to (a reference to item j of attribute runs of text of viewer window 1) --this is a reference to the attribute run in the window.
								exit repeat
							end if
						end repeat
						if theViewRun is missing value then
							tell me to display dialog "Oh Shit! There is no view run that matches the URL!" buttons {"Sigh!"} default button 1
							return
						end if
						-- Strip white space from theText since we will search for a file name that matches it, and we don't want any trailing spaces or LF:
						set theText to current application's NSString's stringWithString_(theText)'s stringByTrimmingCharactersInSet_(current application's NSCharacterSet's whitespaceAndNewlineCharacterSet) as text
						-- Attempt to find an RTF file with that name, but it has to be manually checked and approved later:
						set res to search theText within titles --this lists all items that contain theText in the title, so we need to soon test the thing for an exact match
						if (length of res) is 0 then --sometimes there is a traling dot or parenthesis, so we try to find the file by excluding the last character:
							set res to search (texts 1 thru -2 of theText) within titles
							if (length of res) > 0 then set theText to texts 1 thru -2 of theText
						end if
						set countAnItem to 0 --we want to warn if there are several matches
						repeat with anItem in res
							if kind of anItem is in {"RTFD", "RTF"} and name of anItem is theText then
								set selection of viewer window 1 to {anItem} --this is actually just a proposal. In practice it is the last instance that becomes selected
								set countAnItem to countAnItem + 1
							end if
						end repeat
						if countAnItem > 1 then
							set warningInfo to return & "***Warning: there is more than 1 item with that name!"
						else if countAnItem = 0 then
							set warningInfo to return & "***Warning: no matching file found" & return
						else
							set warningInfo to ""
						end if
						tell me to display dialog ("Link the text \"" & theText & "\"" & return & "in the file named \"" & name of anRTF & "\"" & return & "in the group \"" & name of aGroup & "\"." & return & warningInfo & return & "Select a suitable file that should be the target.") buttons {"Cancel", "Skip", "Proceed"} default button 3
						if button returned of result is "Proceed" then
							set refURL to reference URL of item 1 of (get selection of viewer window 1)
							set selection of viewer window 1 to {anRTF}
							set modDate to modification date of anRTF
							set URL of theViewRun to refURL
							save viewer window 1
							set modification date of anRTF to modDate
							set nrOfChangedLinks to nrOfChangedLinks + 1
							tell me to log "nrOfChangedLinks: " & nrOfChangedLinks --good to know if we want to cancel
							set anRTFisChanged to true
						end if
					end if
				end repeat
				if anRTFisChanged then
					set nrOfChangedFiles to nrOfChangedFiles + 1
					tell me to log "nrOfChangedFiles: " & nrOfChangedFiles --good to know if we want to cancel
				end if
			end repeat
		end if
	end repeat
end tell
return "Number of changes:" & return & ¬
	"Number of changed files: " & nrOfChangedFiles & return & ¬
	"Number of changed links: " & nrOfChangedLinks